﻿using System;
using System.Net;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Threading;

namespace PushService
{
	internal class HttpWebService
	{
		private HttpListener m_server;
		public bool m_isRunning;
		
		public void Start()
		{
			ThreadPool.QueueUserWorkItem((o) => RunServer());
		}

		private void RunServer()
		{
			Int32 port = 8080;
			m_server = new HttpListener();
			m_server.Prefixes.Add(string.Format("http://*:{0}/", port));
			m_server.Start();
			m_isRunning = true;
			while (m_server.IsListening)
			{
				HttpListenerContext ctx = m_server.GetContext();
				ThreadPool.QueueUserWorkItem((object c) => ProcessRequest((HttpListenerContext)c), ctx);
			}
		}

		public void Stop()
		{
			m_isRunning = false;
			if (m_server != null)
			{
				m_server.Stop();
				m_server.Close();
			}
		}

		private void ProcessRequest(HttpListenerContext context)
		{
			switch(context.Request.Url.AbsolutePath)
			{
				case "/registerPush":
					HandlePushRegistration(context);
					break;
				default:
					context.Response.StatusCode = 404;
					break;
			}
			context.Response.OutputStream.Close();
		}

		private void HandlePushRegistration(HttpListenerContext context)
		{
			if (context.Request.HttpMethod == "POST")
			{
				if (context.Request.HasEntityBody)
				{
					System.IO.Stream body = context.Request.InputStream;
					System.Text.Encoding encoding = context.Request.ContentEncoding;
					System.IO.StreamReader reader = new System.IO.StreamReader(body, encoding);
					DataContractJsonSerializer s = new DataContractJsonSerializer(typeof(RegistrationPacket));
					var packet = s.ReadObject(reader.BaseStream) as RegistrationPacket;
					if (packet != null && packet.deviceId != null && !string.IsNullOrWhiteSpace(packet.channelUri))
					{
						if (ClientRegistered != null)
							ClientRegistered(this, packet);
						context.Response.StatusCode = 200;
					}
					else
						context.Response.StatusCode = 500;
				}
				else
					context.Response.StatusCode = 500;
			}
			else
				context.Response.StatusCode = 500;
		}

		/// <summary>
		/// Fired when a device registers itself
		/// </summary>
		public event EventHandler<RegistrationPacket> ClientRegistered;


		[DataContract]
		public class RegistrationPacket
		{
			[DataMember]
			public string channelUri { get; set; }
			[DataMember]
			public string deviceId { get; set; }
		}
	}
}
