﻿using System;
using System.Collections.Generic;

namespace PushService
{
	class Program
	{
		private static Dictionary<string, Uri> registeredClients = new Dictionary<string, Uri>(); //List of registered devices

		static void Main(string[] args)
		{
			//Start http service
			var svc = new HttpWebService();
			svc.Start();
			svc.ClientRegistered += svc_ClientRegistered;

			Console.WriteLine("Service started. Press CTRL-Q to quit.\nPress 'S' to push a message.");
			while (true)
			{
				var key = Console.ReadKey();
				if (key.Key == ConsoleKey.Q && key.Modifiers == ConsoleModifiers.Control)
					break;
				else if(key.Key == ConsoleKey.S)
				{
					Console.WriteLine();
					PushMessageToClients();
				}
			}
			//shut down
			svc.Stop();
		}

		/// <summary>
		/// Called when a client app registers itself with the service
		/// </summary>
		private static void svc_ClientRegistered(object sender, HttpWebService.RegistrationPacket e)
		{
			if(registeredClients.ContainsKey(e.deviceId))
				Console.WriteLine("Client updated: " + e.deviceId);
			else
				Console.WriteLine("Client registered: " + e.deviceId);
			
			registeredClients[e.deviceId] = new Uri(e.channelUri);
		}

		private static void PushMessageToClients()
		{
			if (registeredClients.Count == 0)
				return; //no one to push to

			//Message to send to all clients
			var message = string.Format("{{\"Message\":\"Current server time is: {0}\"}}", DateTime.Now);

			//Generate token for push

			//Set app package SID and client clientSecret (get these for your app from the developer portal)
			string packageSid = "ms-app://s-X-XX-X-XXXXXXXXX-XXXXXXXXXX-XXXXXXXXX-XXXXXXXXXX-XXXXXXXXXX-XXXXXXXXXX-XXXXXXXXXX";
			string clientSecret = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";
			//Generate oauth token required to push messages to client
			OAuthToken accessToken = null;
			try
			{
				accessToken = PushHelper.GetAccessToken(clientSecret, packageSid);
			}
			catch (Exception ex)
			{
				Console.WriteLine("ERROR: Failed to get access token for push : {0}", ex.Message);
				return;
			}

			int counter = 0;
			//Push the message to all the clients
			foreach(var client in registeredClients)
			{
				try
				{
					PushHelper.Push(client.Value, accessToken, message);
					counter++;
				}
				catch (Exception ex)
				{
					Console.WriteLine("ERROR: Failed to push to {0}: {1}", client.Key, ex.Message);
				}
			}
			Console.WriteLine("Pushed successfully to {0} client(s)", counter);
		}
	}
}
