﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.Networking.PushNotifications;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace PushTestApp
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();
        }

		private async void ButtonRegister_Click(object sender, RoutedEventArgs e)
		{
			var channel = await PushNotificationChannelManager.CreatePushNotificationChannelForApplicationAsync();
			var uri = channel.Uri;
			channel.PushNotificationReceived += channel_PushNotificationReceived;

			RegisterWithServer(uri);
		}

		private async void RegisterWithServer(string uri)
		{
			string IP = "192.168.1.17"; //IP address of server
			HttpClient client = new HttpClient();

			DataContractJsonSerializer s = new DataContractJsonSerializer(typeof(RegistrationPacket));
			RegistrationPacket packet = new RegistrationPacket()
			{
				channelUri = uri,
				deviceId = GetHardwareId()
			};
			System.IO.MemoryStream ms = new System.IO.MemoryStream();
			s.WriteObject(ms, packet);
			ms.Seek(0, System.IO.SeekOrigin.Begin);
			try
			{
				var result = await client.PostAsync(new Uri("http://" + IP + ":8080/registerPush"), new StreamContent(ms));
				Status.Text = "Push registration successfull";
			}
			catch(System.Exception ex) {
				Status.Text = "Push registration failed: " + ex.Message;
			}
		}

		private string GetHardwareId()
		{
			var token = Windows.System.Profile.HardwareIdentification.GetPackageSpecificToken(null);
			var hardwareId = token.Id;
			var dataReader = Windows.Storage.Streams.DataReader.FromBuffer(hardwareId);

			byte[] bytes = new byte[hardwareId.Length];
			dataReader.ReadBytes(bytes);
			return BitConverter.ToString(bytes);
		}

		[DataContract]
		public class RegistrationPacket
		{
			[DataMember]
			public string channelUri { get; set; }
			[DataMember]
			public string deviceId { get; set; }
		}


		private void channel_PushNotificationReceived(PushNotificationChannel sender, PushNotificationReceivedEventArgs args)
		{
			var content = args.RawNotification.Content;
			var _ = Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () =>
			{
				Status.Text = "Received: " + content;
			});
		}
    }
}
